// Tide Protocol - Infrastructure for the Personal Data economy
// Copyright (C) 2019 Tide Foundation Ltd
// 
// This program is free software and is subject to the terms of 
// the Tide Community Open Source License as published by the 
// Tide Foundation Limited. You may modify it and redistribute 
// it in accordance with and subject to the terms of that License.
// This program is distributed WITHOUT WARRANTY of any kind, 
// including without any implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE.
// See the Tide Community Open Source License for more details.
// You should have received a copy of the Tide Community Open 
// Source License along with this program.
// If not, see https://tide.org/licenses_tcosl-1-0-en
import { randomBytes } from 'crypto';
import Hash from './hash';
import AESKey from './aesKey';

export default class AesSherableKey extends AESKey {
    /**
     * @param {Uint8Array} key
     * @param {Uint8Array} mac
     */
    constructor(key = null, mac = null) {
        if (key === null && mac === null)
            [key, mac] = genKeys(randomBytes(32));
        
        super(key, mac);
    }

    toString() {
        return Buffer.from(this.toArray()).toString("base64");
    }

    /** @returns {Uint8Array} */
    toArray() {
        const buffer = Buffer.alloc(32);
        buffer.fill(this.key, 0, 16);
        Buffer.from(this.mac).copy(buffer, 16, 0, 16);

        return buffer;
    }

    /** @param {string | Uint8Array} data */
    static from(data) {
        const [key, mac] = genKeys(typeof data === "string" ? 
            Buffer.from(data, "base64") : Buffer.from(data));

        return new AesSherableKey(key, mac);
    }
}

/** @param {Buffer} key */
function genKeys(key) {
    const mac = Hash.shaBuffer(key);
    key.copy(mac, 0, 16, 32);
    return [key.slice(0, 16), mac];
}