import assert from 'assert';
import ed25519Key from '../src/ed25519Key.js';
import { randomBytes } from 'crypto';

describe('ed25519Key', function () {
    it('should serialize and deserialize', function () {
        for (let i = 0; i < 20; i++) {
            var key1 = ed25519Key.generate();
            var key2 = ed25519Key.from(key1.toString())

            assert.equal(key1.toString(), key2.toString());
        }
    });

    it('should serialize and deserialize public', function () {
        for (let i = 0; i < 20; i++) {
            var key1 = ed25519Key.public(randomBytes(32));
            var key2 = ed25519Key.from(key1.toString())

            assert.equal(key1.toString(), key2.toString());
        }
    });

    it('should serialize and deserialize private', function () {
        for (let i = 0; i < 20; i++) {
            var key1 = ed25519Key.private(randomBytes(32));
            var key2 = ed25519Key.from(key1.toString())

            assert.equal(key1.toString(), key2.toString());
        }
    });

    it('should sign and verify bytes', function () {
        for (let i = 0; i < 20; i++) {
            var key = ed25519Key.generate();
            var pub = key.pub;
            var message = 'hi';

            var signature = key.sign(message);
            var isOk = pub.verify(message, signature);

            assert.equal(true, isOk);
        }
    });
});
